@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 1, 2017
+# Last Change: July 10, 2017
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -54,11 +54,34 @@ __all__ = (
     'PatchedItem',
     'TemporaryDirectory',
     'TestCase',
+    'configure_logging',
     'retry',
     'run_cli',
 )
 
 
+def configure_logging(log_level=logging.DEBUG):
+    """configure_logging(log_level=logging.DEBUG)
+    Automatically configure logging to the terminal.
+
+    :param log_level: The log verbosity (a number, defaults to
+                      :data:`logging.DEBUG`).
+
+    When :mod:`coloredlogs` is installed :func:`coloredlogs.install()` will be
+    used to configure logging to the terminal. When this fails with an
+    :exc:`~exceptions.ImportError` then :func:`logging.basicConfig()` is used
+    as a fall back.
+    """
+    try:
+        import coloredlogs
+        coloredlogs.install(level=log_level)
+    except ImportError:
+        logging.basicConfig(
+            level=log_level,
+            format='%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s',
+            datefmt='%Y-%m-%d %H:%M:%S')
+
+
 def retry(func, timeout=60, exc_type=AssertionError):
     """retry(func, timeout=60, exc_type=AssertionError)
     Retry a function until assertions no longer fail.
@@ -118,22 +141,50 @@ def run_cli(entry_point, *arguments, **options):
               1. The return code (an integer).
               2. The captured output (a string).
     """
-    program_name = options.pop('program_name', sys.executable)
-    # Start capturing the standard output and error streams.
-    with CaptureOutput(**options) as output:
+    # Add the `program_name' option to the arguments.
+    arguments = list(arguments)
+    arguments.insert(0, options.pop('program_name', sys.executable))
+    # Log the command line arguments (and the fact that we're about to call the
+    # command line entry point function).
+    logger.debug("Calling command line entry point with arguments: %s", arguments)
+    # Prepare to capture the return code and output even if the command line
+    # interface raises an exception (whether the exception type is SystemExit
+    # or something else).
+    returncode = 0
+    stdout = None
+    stderr = None
+    try:
         # Temporarily override sys.argv.
-        arguments = list(arguments)
-        arguments.insert(0, program_name)
         with PatchedAttribute(sys, 'argv', arguments):
-            try:
-                # Call the command line interface.
-                entry_point()
-            except SystemExit as e:
-                # Intercept sys.exit() calls.
-                returncode = e.code
+            # Manipulate the standard input/output/error streams.
+            with CaptureOutput(**options) as capturer:
+                try:
+                    # Call the command line interface.
+                    entry_point()
+                finally:
+                    # Get the output even if an exception is raised.
+                    stdout = capturer.stdout.getvalue()
+                    stderr = capturer.stderr.getvalue()
+                    # Reconfigure logging to the terminal because it is very
+                    # likely that the entry point function has changed the
+                    # configured log level.
+                    configure_logging()
+    except BaseException as e:
+        if isinstance(e, SystemExit):
+            logger.debug("Intercepting return code %s from SystemExit exception.", e.code)
+            returncode = e.code
+        else:
+            logger.warning("Defaulting return code to 1 due to raised exception.", exc_info=True)
+            returncode = 1
+    else:
+        logger.debug("Command line entry point returned successfully!")
+    finally:
+        for name, value in (('stdout', stdout), ('stderr', stderr)):
+            if value:
+                logger.debug("Output on %s:\n%s", name, value)
             else:
-                returncode = 0
-            return returncode, output.getvalue()
+                logger.debug("No output on %s.", name)
+        return returncode, stdout
 
 
 class CallableTimedOut(Exception):
@@ -284,6 +335,42 @@ class TemporaryDirectory(ContextManager):
             self.temporary_directory = None
 
 
+class MockedHomeDirectory(PatchedItem, TemporaryDirectory):
+
+    """
+    Context manager to temporarily change ``$HOME`` (the current user's profile directory).
+
+    This class is a composition of the :class:`PatchedItem` and
+    :class:`TemporaryDirectory` context managers.
+    """
+
+    def __init__(self):
+        """Initialize a :class:`MockedHomeDirectory` object."""
+        PatchedItem.__init__(self, os.environ, 'HOME', os.environ.get('HOME'))
+        TemporaryDirectory.__init__(self)
+
+    def __enter__(self):
+        """
+        Activate the custom ``$PATH``.
+
+        :returns: The pathname of the directory that has
+                  been added to ``$PATH`` (a string).
+        """
+        # Get the temporary directory.
+        directory = TemporaryDirectory.__enter__(self)
+        # Override the value to patch now that we have
+        # the pathname of the temporary directory.
+        self.patched_value = directory
+        # Temporary patch $HOME.
+        PatchedItem.__enter__(self)
+        # Pass the pathname of the temporary directory to the caller.
+        return directory
+
+    def __exit__(self, exc_type=None, exc_value=None, traceback=None):
+        """Deactivate the custom ``$HOME``."""
+        super(MockedHomeDirectory, self).__exit__(exc_type, exc_value, traceback)
+
+
 class CustomSearchPath(PatchedItem, TemporaryDirectory):
 
     """
@@ -482,17 +569,14 @@ class TestCase(unittest.TestCase):
         """setUp(log_level=logging.DEBUG)
         Automatically configure logging to the terminal.
 
-        :param log_level: The log verbosity (a number, defaults to
-                          :data:`logging.DEBUG`).
+        :param log_level: Refer to :func:`configure_logging()`.
 
         The :func:`setUp()` method is automatically called by
         :class:`unittest.TestCase` before each test method starts.
         It does two things:
 
-        - When :mod:`coloredlogs` is installed :func:`coloredlogs.install()`
-          will be used to configure logging to the terminal. When this fails
-          with an :exc:`~exceptions.ImportError` then
-          :func:`logging.basicConfig()` is used as a fall back.
+        - Logging to the terminal is configured using
+          :func:`configure_logging()`.
 
         - Before the test method starts a newline is emitted, to separate the
           name of the test method (which will be printed to the terminal by
@@ -500,14 +584,7 @@ class TestCase(unittest.TestCase):
           that the test method is likely going to generate.
         """
         # Configure logging to the terminal.
-        try:
-            import coloredlogs
-            coloredlogs.install(level=log_level)
-        except ImportError:
-            logging.basicConfig(
-                level=log_level,
-                format='%(asctime)s %(name)s[%(process)d] %(levelname)s %(message)s',
-                datefmt='%Y-%m-%d %H:%M:%S')
+        configure_logging(log_level)
         # Separate the name of the test method (printed by the superclass
         # and/or py.test without a newline at the end) from the first line of
         # logging output that the test method is likely going to generate.
